""" Command Line Interface (CLI) module """
from os import path
from sys import argv
from argparse import ArgumentParser, Namespace

from . import JSON_Format
from .core import BaseLang

try:
    from . import YAML_Format
    yaml_imported = True
except ImportError:
    yaml_imported = False

try:
    from . import TOML_Format
    toml_imported = True
except ImportError:
    toml_imported = False


def convert(from_path: str, from_format: str, dest_path: str, dest_format: str):
    """Convert files between any supported languages

    :param from_path: Path to the file to be converted
    :type from_path: str
    :param from_format: format of the file to be converted
    :type from_format: str
    :param dest_path: desired path to converted file
    :type dest_path: str
    :param dest_format: desired file format
    :type dest_format: str
    """
    from_object = __generate_serialix_object(from_path, from_format)
    dest_object = __generate_serialix_object(dest_path, dest_format)

    if from_object is not None and dest_object is not None:
        if not from_object.is_file_exist():
            print('local file on path "{}" does not exist, nothing to convert'
                  .format(from_object.local_file_path)
                  )
            exit()

        dest_object.dictionary = from_object.dictionary
        dest_object.commit()

        print('successfully converted "{}" from "{}" to "{}" and saved the result to "{}"'.format(
            path.basename(from_path),
            from_format,
            dest_format,
            dest_path
        ))


def __parse_args() -> Namespace:
    """Parse all input cli arguments with ``argparse`` module

    :return: Namespace with all arguments
    :rtype: Namespace
    """
    parser_main = ArgumentParser(
        description='command line interface toolset for "serialix"'
    )
    subparsers = parser_main.add_subparsers(dest='command')

    parser_converter = subparsers.add_parser('convert', help='tool for conversion between supported languages')
    parser_converter.add_argument('from_path', action='store', type=str, help='path to the file to be converted')
    parser_converter.add_argument('from_format', action='store', type=str, help='format of the file to be converted')
    parser_converter.add_argument('dest_path', action='store', type=str, help='desired path to converted file')
    parser_converter.add_argument('dest_format', action='store', type=str, help='desired file format')

    if len(argv) <= 1:
        parser_main.print_help()
        exit()

    return parser_main.parse_args()


def __generate_serialix_object(file_path: str, file_lang: str) -> BaseLang:
    """Automatic generation of ``serialix`` object

    :param file_path: Path to file that will be used in object
    :type file_path: str
    :param file_lang: Language of this file
    :type file_lang: str
    :return: ``*_Format`` class object
    :rtype: BaseLang
    """
    obj = None

    if file_lang == 'json':
        obj = JSON_Format(file_path, auto_file_creation=False)
    elif file_lang == 'yaml' and yaml_imported:
        obj = YAML_Format(file_path, auto_file_creation=False)
    elif file_lang == 'toml' and toml_imported:
        obj = TOML_Format(file_path, auto_file_creation=False)
    else:
        print('language "{}" for file "{}" is not supported'.format(file_lang, file_path))

    return obj


def main_cli(args: Namespace):
    """Main cli handler that processes the input arguments

    :param args: Arguments to be parsed
    :type args: Namespace
    """
    if args.command == 'convert':
        convert(
            args.from_path, args.from_format,
            args.dest_path, args.dest_format
        )


def start():
    """
    Startup function that will read the input arguments and pass them
    to main cli handler. Used by module and entry point calls.
    """
    main_cli(__parse_args())


if __name__ == "__main__":
    start()



import pytest

from serialix.core import create_directories

from . import core


core.change_path_to_testsdir()


def pytest_configure(config):
    custom_markers = (
        'json: run only json tests',
        'yaml: run only yaml tests',
        'toml: run only toml tests',
    )

    for marker in custom_markers:
        config.addinivalue_line(
            'markers', marker
        )


@pytest.fixture(scope='class', autouse=True)
def dir_control():
    create_directories(core.temp_dir_path)
    yield 'class'
    core.remove_temp_dir()



""" ``JSON`` language support """
from ..core import BaseLang

try:
    import ujson as json
except ImportError:
    import json


class JSON_Format(BaseLang):
    """JSON Data Interchange Format (DIF) realisation

    :param file_path: Path to preferred local file destination
        If the file does not exist at the specified path, it will be created
    :type file_path: str
    :param default_dictionary: Default local file path ``str`` or ``dict``
        that will be used for local file start values and , defaults to {}
    :type default_dictionary: Union[str, dict], optional
    :param auto_file_creation: Automatic local file creation on object initialization, defaults to True
    :type auto_file_creation: bool, optional
    :param force_overwrite_file: Whether the file needs to be overwritten if it already exists, defaults to False
    :type force_overwrite_file: bool, optional
    :param parser_write_kwargs: Pass custom arguments to parser's write to local file action, defaults to {}
    :type parser_write_kwargs: dict, optional
    :param parser_read_kwargs: Pass custom arguments to parser's read from local file action, defaults to {}
    :type parser_read_kwargs: dict, optional
    :raises ValueError: If provided data type in argument ``default_dictionary`` is not
        the path ``str`` or ``dict``, this exception will be raised

    .. note::
        Methods ``.clear()``, ``.fromkeys()``, ``.get()``, ``.items()``, ``.keys()``, ``values()``,
        ``pop()``, ``popitem()``, ``setdefault()``, ``update()`` are bound to the attribute ``dictionary``,
        so executing:

        >>> this_object.update({'check': True})

        Is equal to:

        >>> this_object.dictionary.update({'check': True})
    """
    def _core__read_file_to_dict(self, file_path: str) -> dict:
        """Method for reading custom local files from path ``str`` as dictionary

        :param file_path: Path to local file in ``json`` format
        :type file_path: str
        :return: Parsed local file dictionary
        :rtype: dict
        """
        with open(file_path, 'rt') as f:
            config_dict = json.load(f, **self.parser_read_kwargs)

        return config_dict

    def _core__write_dict_to_file(self, file_path: str, dictionary: dict):
        """Method to write dictionaries into custom local path ``str``

        :param file_path: Path to local file in ``json`` format
        :type file_path: str
        :param dictionary: Dictionary which will be written in ``file_path``
        :type dictionary: dict
        """
        with open(file_path, 'wt') as f:
            json.dump(dictionary, f, **self.parser_write_kwargs)



"""
Here is the main initialization code that makes
it easier to access the main features of the
other submodules and subpackages
"""
from .meta import *
from .langs.json import JSON_Format

try:
    from .langs.yaml import YAML_Format
except ImportError as e:
    pass

try:
    from .langs.toml import TOML_Format
except ImportError:
    pass



from typing import Any
from copy import deepcopy
from ast import literal_eval
from os import path, makedirs, remove


class BaseLang:
    """
    Base Data Interchange Format (DIF) realisation.

    This class is not intended to be used directly. Instead, it should be inherited by a class, that provides the support
    for DIF language. Inherited class must overwrite the ``_core__read_file_to_dict()`` and ``_core__write_dict_to_file()``
    static methods with real implementation of their functionality that described in the docstring of each method.

    :param file_path: Path to preferred local file destination
        If the file does not exist at the specified path, it will be created
    :type file_path: str
    :param default_dictionary: Default local file path ``str`` or ``dict``
        that will be used for local file start values and , defaults to {}
    :type default_dictionary: Union[str, dict], optional
    :param auto_file_creation: Automatic local file creation on object initialization, defaults to True
    :type auto_file_creation: bool, optional
    :param force_overwrite_file: Whether the file needs to be overwritten if it already exists, defaults to False
    :type force_overwrite_file: bool, optional
    :param parser_write_kwargs: Pass custom arguments to parser's write to local file action, defaults to {}
    :type parser_write_kwargs: dict, optional
    :param parser_read_kwargs: Pass custom arguments to parser's read from local file action, defaults to {}
    :type parser_read_kwargs: dict, optional
    :raises ValueError: If provided data type in argument ``default_dictionary`` is not
        the path ``str`` or ``dict``, this exception will be raised

    .. note::
        Methods ``.clear()``, ``.fromkeys()``, ``.get()``, ``.items()``, ``.keys()``, ``values()``,
        ``pop()``, ``popitem()``, ``setdefault()``, ``update()`` are bound to the attribute ``dictionary``,
        so executing:

        >>> this_object.update({'check': True})

        Is equal to:

        >>> this_object.dictionary.update({'check': True})
    """
    def __init__(self, file_path: str, default_dictionary={}, auto_file_creation=True, force_overwrite_file=False, parser_write_kwargs={}, parser_read_kwargs={}):
        self.__parsed_dict = {}
        self.local_file_path = file_path
        self.parser_write_kwargs = parser_write_kwargs
        self.parser_read_kwargs = parser_read_kwargs

        if isinstance(default_dictionary, dict):
            self.__default_dict = default_dictionary
        elif path.isfile(default_dictionary):
            self.__default_dict = self._core__read_file_to_dict(default_dictionary)
        else:
            raise ValueError('"default_dictionary" argument should be a dictionary or a path to file string. Provided value is {0}'
                             .format(default_dictionary))

        if auto_file_creation:
            if not self.is_file_exist() or force_overwrite_file:
                self.create_file()

        self.reload()

    def __getitem__(self, key):
        return self.__parsed_dict[key]

    def __setitem__(self, key, value):
        self.__parsed_dict[key] = value

    def __delitem__(self, key):
        del(self.__parsed_dict[key])

    def __len__(self):
        return self.__parsed_dict.__len__()

    def __iter__(self):
        return self.__parsed_dict.__iter__()

    def clear(self):
        """ Clear the bound ``dictionary`` """
        self.__parsed_dict.clear()

    def copy(self, deep_mode=True) -> dict:
        """Get the copy of ``dictionary``.
        This method uses the recursive copy function that will remove
        all references to original dictionary.

        :param deep_mode: Use the recursive copy function to remove
            all references to original dictionary. Disabling this will lead
            to saving the references of the nested dictionaries to the copy object,
            defaults to True.
        :type deep_mode: bool, optional
        :return: ``dictionary`` copy
        :rtype: dict
        """
        if deep_mode:
            return deepcopy(self.__parsed_dict)
        else:
            return self.__parsed_dict.copy()

    def get(self, key, default=None) -> Any:
        """Get key from ``dictionary``

        :param key: Key name
        :type key: str
        :param default: Default value, if key was not found, defaults to None
        :type default: Any
        :return: Value of requested ``key``, or ``default`` value
            if key wasn't found.
        :rtype: Any
        """
        return self.__parsed_dict.get(key, default)

    def items(self) -> list:
        """Get items of the ``dictionary``

        :return: Items of the ``dictionary`` ((key, value) pairs)
        :rtype: list
        """
        return self.__parsed_dict.items()

    def keys(self) -> list:
        """Get keys of the ``dictionary``

        :return: Keys of the ``dictionary`` (, key)
        :rtype: list
        """
        return self.__parsed_dict.keys()

    def values(self) -> list:
        """Get values of the ``dictionary``

        :return: Values of the ``dictionary`` (, value)
        :rtype: list
        """
        return self.__parsed_dict.values()

    def pop(self, key: Any, default=None) -> Any:
        """Pop key from ``dictionary``

        :param key: Key name
        :type key: Any
        :param default: Default value, if key was not found, defaults to None
        :type default: Any, optional
        :return: Value of requested ``key``, or ``default`` value
            if key wasn't found, defaults to None.
        :rtype: Any
        """
        return self.__parsed_dict.pop(key, default)

    def popitem(self) -> Any:
        """Pop item from ``dictionary`` in LIFO order.

        :param key: Key name
        :type key: Any
        :param default: Default value, if key was not found, defaults to None
        :type default: Any, optional
        :return: Value of requested ``key``, or ``default`` value
            if key wasn't found, defaults to None.
        :rtype: Any
        """
        return self.__parsed_dict.popitem()

    def setdefault(self, key: Any, default=None) -> Any:
        """
        If key is in the ``dictionary``, return its value.
        If not, insert key with a value of ``default`` and return ``default``

        :param key: Name of the key
        :type key: Any
        :param default: Default value, defaults to None
        :type default: Any, optional
        :return: If key is in the dictionary, return its value, else:
            returns ``defalut``
        :rtype: Any
        """
        return self.__parsed_dict.setdefault(key, default)

    def update(self, dictionary: dict):
        """Update ``dictionary`` with another dictionary

        :param dictionary: Dictionary, that will be merged to
            ``dictionary``
        :type dictionary: dict
        """
        self.__parsed_dict.update(dictionary)

    @property
    def dictionary(self) -> dict:
        """Full access to the dictionary attribute.
        Contains local file data parsed to dictionary

        :return: ``dictionary`` attribute
        :rtype: dict
        """
        return self.__parsed_dict

    @dictionary.setter
    def dictionary(self, dictionary: dict):
        self.__parsed_dict = dictionary

    @property
    def dictionary_default(self) -> dict:
        """Full access to the default dictionary.
        Contains dictionary with default keys that was
        specified in ``default_dictionary`` argument.

        :return: default dictionary
        :rtype: dict
        """
        return self.__default_dict

    @dictionary_default.setter
    def dictionary_default(self, dictionary: dict):
        self.__default_dict = dictionary

    def commit(self):
        """Commit all changes from ``dictionary`` to local file"""
        self.write_dict_to_file(self.__parsed_dict)

    def refresh(self, safe_mode=True) -> bool:
        """
        Refresh ``dictionary`` values from local file.
        Note that this method does not remove user-added keys,
        it will only add non existent keys and modify the already existing keys.

        :param safe_mode: Provides the recursive merge of dictionary from local
            file to object's ``dictionary``. This option prevents object's nested
            dictionaries to be overwritten by local files, but is much slower than
            simple ``dict.update()`` method call. So if you don't care about nested
            dictionaries be overwritten, you can disable this feature to boost the
            execution speed
        :type safe_mode: bool, optional
        :return: Status of local file read action. If file does not exist -
            ``False`` will be returned.
        :rtype: bool
        """
        if not self.is_file_exist():
            return False

        if safe_mode:
            self.__parsed_dict = recursive_dicts_merge(self.read_file_as_dict(), self.__parsed_dict)
        else:
            self.__parsed_dict.update(self.read_file_as_dict())

        return True

    def reload(self) -> bool:
        """Reset the ``dictionary`` attribute to values from local file

        :return: Status of local file read action. If file does not exist -
            ``False`` will be returned.
        :rtype: bool
        """
        if self.is_file_exist():
            self.__parsed_dict = self.read_file_as_dict()
            return True
        else:
            return False

    def reset_to_defaults(self):
        """
        Reset the ``dictionary`` attribute to values from ``dictionary_default`` attribute.
        Note that local file will stay untouched.
        """
        self.__parsed_dict = deepcopy(self.__default_dict)

    def create_file(self) -> bool:
        """Create new local file from default dictionary

        :return: Was the file created successfully
        :rtype: bool
        """
        self.write_dict_to_file(self.__default_dict)

        return True

    def delete_file(self) -> bool:
        """Delete local file

        :return: Was the file removed.
            False will be returned only if the local file does not exist at the time of deletion.
        :rtype: bool
        """
        if self.is_file_exist():
            remove(self.local_file_path)
            return True
        else:
            return False

    def is_file_exist(self) -> bool:
        """Check local file existence

        :return: Does the file exist
        :rtype: bool
        """
        return path.isfile(self.local_file_path)

    def write_dict_to_file(self, dictionary: dict):
        """Write dict from ``dictionary`` argument to local file bound to this object

        :param dictionary: Dictionary that should be written to file
        :type dictionary: dict
        """
        create_directories(self.local_file_path)
        self._core__write_dict_to_file(self.local_file_path, dictionary)

    def read_file_as_dict(self) -> dict:
        """Read local file bound to this object as dictionary

        :return: Parsed to dictionary local file
        :rtype: dict
        """
        return self._core__read_file_to_dict(self.local_file_path)

    def _core__read_file_to_dict(self, file_path: str) -> dict:
        """Template for reading custom local files from path ``str`` as dictionary

        :param file_path: Path to local file
        :type file_path: str
        :return: Parsed local file dictionary
        :rtype: dict
        """
        pass

    def _core__write_dict_to_file(self, file_path: str, dictionary: dict):
        """Template for writing dictionaries into custom local path ``str``

        :param file_path: Path to local file
        :type file_path: str
        :param dictionary: Dictionary which will be written in ``file_path``
        :type dictionary: dict
        """
        pass


def create_directories(path_to_use: str, path_is_dir=False):
    """Create all directories from path

    :param path_to_use: The path to be created
    :type path_to_use: str
    :param path_is_dir: Is ``path_to_use`` ends with directory, defaults to False
    :type path_is_dir: bool, optional
    """
    path_to_use = path_to_use if path_is_dir else path.dirname(path_to_use)

    if not path.exists(path_to_use) and len(path_to_use) > 0:
        makedirs(path_to_use)


def recursive_dicts_merge(merge_from: dict, merge_to: dict) -> dict:
    """
    This function will recursively merge ``merge_from`` dictionary to ``merge_to``.
    Merging with this function, instead of the ``dict.update()`` method prevents from
    keys removal of nested dictionaries.

    :param merge_from: Dictionary to merge keys from
    :type merge_from: dict
    :param merge_to: Dictionary to merge keys to
    :type merge_to: dict
    :return: Dictionary with all ``merge_from`` keys merged into ``merge_to``
    :rtype: dict

    .. note::
        ``merge_from`` and ``merge_to`` dictionaries will not be modified in process of execution.
        Function will get their copies and work with them.
    """
    def __merge(merge_from: dict, merge_to: dict):
        for k, v in merge_from.items():
            if isinstance(v, dict):
                __merge(v, merge_to.setdefault(k, {}))
            else:
                merge_to[k] = v

    merge_from = deepcopy(merge_from)
    result_dict = deepcopy(merge_to)

    __merge(merge_from, result_dict)

    return result_dict


def parse_dict_values(input_dict: dict) -> dict:
    """
    This function is written for DIF parsers and languages, that doesn't support
    storing values in their original types, converting them all to ``str`` type.
    Function will recursively scan the input dictionary and will try to convert all
    keys with values which type is ``str``

    :param input_dict: [description]
    :type input_dict: dict
    :return: [description]
    :rtype: dict
    """
    def __recursive_safe_eval(inputd: dict, outputd: dict):
        # TODO
        for k, v in inputd.items():
            if not isinstance(v, str): continue

            try:
                evaled_v = literal_eval(v)
            except ValueError:
                continue
            else:
                if isinstance(evaled_v, dict):
                    __recursive_safe_eval(v, outputd.setdefault(k, {}))
                else:
                    outputd[k] = evaled_v

    result = {}
    __recursive_safe_eval(input_dict, result)

    return result



from shutil import rmtree
from os import path, chdir

from serialix.core import BaseLang


temp_dir_path = './temp/'

local_file_path = path.join(temp_dir_path, 'test_config.txt')
default_cfg_path = path.join(temp_dir_path, 'default_config.txt')

default_local_file_dict = {
    'root': {
        'Person_1': {
            'name': 'X',
            'age': 256
        },
        'case_415': {
            'path': '~/xxx/xxx/xxx',
            'utime': 1603645369,
            'description': 'He was lying on the floor in tears with a keyboard in his hand. '
                           'The light of the monitor illuminated the whole sad sight. '
                           'He was just trying to write unit tests.'
        },
        'TestKey': 'Yes',
        'dash-split': {
            'wow-man': 'yeah, cool'
        }
    }
}


def remove_temp_dir():
    if path.isdir(temp_dir_path):
        rmtree(temp_dir_path)


def change_path_to_testsdir():
    """ Change working directory to `/tests/` """
    chdir(path.dirname(__file__))


class BaseLangTest():
    """
    Base class for all tests.
    `setup_method()` method should be overwritten with valid object initialization.
    """
    def setup(self):
        self.language_object = BaseLang()

    def test_delete_file(self):
        """
        Delete local file and check
        is the functions output works correctly
        """
        expect_success = self.language_object.delete_file()
        expect_fail = not self.language_object.delete_file()

        assert expect_success and expect_fail

    def test_create_file(self):
        """ Create local file """
        self.language_object.create_file()

        assert self.language_object.is_file_exist()

    def test_file_read(self):
        assert self.language_object.read_file_as_dict()['root']['Person_1']['name'] == default_local_file_dict['root']['Person_1']['name']

    def test_commit_changes(self):
        person_name = 'Jeff'
        self.language_object['root']['Person_1']['name'] = person_name
        self.language_object.commit()

        read_dict = self.language_object.read_file_as_dict()

        assert read_dict['root']['Person_1']['name'] == person_name

    def test_clear_dict(self):
        """ Clear the `dictionary` attribute """
        self.language_object.clear()

        assert len(self.language_object) == 0

    def test_refresh_safe(self):
        """
        Refresh `dictionary` from local file in safe mode
        and check for nested dictionary existance inside.
        """
        self.language_object['root']['refresh_safe_test'] = {'nested_key_1': 1}
        self.language_object.commit()
        self.language_object['root']['refresh_safe_test'] = {'nested_key_2': 2}

        self.language_object.refresh(safe_mode=True)

        assert self.language_object['root']['refresh_safe_test'].get('nested_key_1', 0) == 1 and \
               self.language_object['root']['refresh_safe_test'].get('nested_key_2', 0) == 2

    def test_refresh_unsafe(self):
        self.language_object['root']['nested_dict_refresh_test'] = {'will_stay': 'cia'}
        self.language_object.commit()
        self.language_object['root']['nested_dict_refresh_test']['will_be_lost'] = 'what!?'
        self.language_object.refresh(safe_mode=False)

        assert self.language_object['root']['nested_dict_refresh_test'].get('will_be_lost', None) is None and \
               self.language_object['root']['nested_dict_refresh_test'].get('will_stay', None) is not None

    def test_reload_dict(self):
        """ Reset `dictionary` to values from bound local file """
        self.language_object['root']['tobereset'] = False
        self.language_object.reload()

        assert self.language_object.get('tobereset', True)

    def test_access_dashed_attribute(self):
        assert self.language_object['root']['dash-split']['wow-man'] == default_local_file_dict['root']['dash-split']['wow-man']

    def test_add_dashed_attribute(self):
        self.language_object['root']['dash-split']['second-dashed'] = 'ok guys, bye'
        self.language_object.commit()

        assert self.language_object['root']['dash-split']['second-dashed'] == \
               self.language_object.read_file_as_dict()['root']['dash-split']['second-dashed']

    def test_add_one_attribute(self):
        """
        Add one new attribute to `dictionary`, commit changes
        and assert the local file to `dictionary`
        """
        key_name = 'Masvingo'

        self.language_object['root']['city'] = key_name
        self.language_object.commit()

        assert self.language_object['root']['city'] == self.language_object.read_file_as_dict()['root']['city']

    def test_add_nested_dicts(self):
        """
        Add nested dictionary to `dictionary`, commit changes and assert
        local file values to `dictionary`
        """
        dct = {
            'sugar': {
                'carbon': 12,
            }
        }

        self.language_object['root']['items'] = dct
        self.language_object.commit()

        assert self.language_object['root']['items']['sugar']['carbon'] == self.language_object.read_file_as_dict()['root']['items']['sugar']['carbon']

    def test_add_multiple_dicts_with_update(self):
        """
        Add multiple dicts to `dictionary`, commit changes
        and assert the result with local file
        """
        dct = {
            'jerre_1829': {
                'status': 1
            },
            'wolfrm4882': {
                'status': 2
            }
        }

        self.language_object['root']['users'] = dct
        self.language_object.commit()

        file_dict = self.language_object.read_file_as_dict()['root']['users']

        print(file_dict['jerre_1829']['status'])
        print(self.language_object['root']['users']['jerre_1829']['status'])

        assert self.language_object['root']['users']['jerre_1829']['status'] == file_dict['jerre_1829']['status'] and \
               self.language_object['root']['users']['wolfrm4882']['status'] == file_dict['wolfrm4882']['status']

    def test_modify_default_dict(self):
        """Modify the default dictionary, reset `dictionary` to defaults and assert values"""
        self.language_object.dictionary_default['root'].update({'new_values': [256, 45, 154]})
        self.language_object.reset_to_defaults()

        assert self.language_object['root'].get('new_values', None) == self.language_object.dictionary_default['root']['new_values']



import json

import pytest

from serialix import JSON_Format

from .core import BaseLangTest, local_file_path,\
    default_local_file_dict, default_cfg_path


@pytest.mark.json
class Test_Create_DefDict(BaseLangTest):
    """
    Create local file on `local_file_path`
    and get default config values from `default_local_file_dict`
    """
    def setup(self):
        self.language_object = JSON_Format(
            local_file_path,
            default_local_file_dict
        )


@pytest.mark.json
class Test_Create_DefPath(BaseLangTest):
    """
    Read existing file on `local_file_path`
    and get default config values from file on `default_cfg_path`
    """
    def setup(self):
        with open(default_cfg_path, 'w') as f:
            json.dump(default_local_file_dict, f)

        self.language_object = JSON_Format(
            local_file_path,
            default_cfg_path
        )



from setuptools import setup, find_packages


def get_package_version() -> str:
    nspace = {}

    with open('./serialix/meta.py', 'r') as f:
        exec(f.read(), nspace)

    return nspace['__version__']


package_name = 'serialix'
package_version = get_package_version()

# Form extras
extras_require = {
    'ujson': ['ujson<=4.0.1'],
    'yaml': ['ruamel.yaml<=0.16.12'],
    'toml': ['toml<=0.10.2']
}

all_base_requirements = [dep for v in extras_require.values() for dep in v]

#  Installation with all parsers
extras_require.update({
    'full': all_base_requirements
})

#  Installation for testing
extras_require.setdefault('test', all_base_requirements.copy()).append('pytest<7')

with open('README.rst', 'r') as f:
    readme_text = f.read()

setup(
    name=package_name,
    version=package_version,
    python_requires='~=3.5',
    author='maximilionus',
    author_email='maximilionuss@gmail.com',
    description='Powerful and easy to use tool for working with various data interchange formats (json, yaml, etc.)',
    long_description_content_type='text/x-rst',
    long_description=readme_text,
    keywords='data interchange format files dif json yaml toml',
    packages=find_packages(),
    extras_require=extras_require,
    entry_points={
        'console_scripts': [
            'serialixcli = serialix.cli:start'
        ]
    },
    license='MIT',
    url='https://github.com/maximilionus/serialix',
    project_urls={
        'Documentation': 'https://maximilionus.github.io/serialix',
        'Tracker': 'https://github.com/maximilionus/serialix/issues'
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries'
    ]
)
